@@ -11,7 +11,7 @@ from collections import defaultdict
 import requests
 
 from spotipy.exceptions import SpotifyException
-from spotipy.util import Retry
+from spotipy.util import REQUESTS_SESSION, Retry
 
 logger = logging.getLogger(__name__)
 
@@ -211,11 +211,8 @@ class Spotify:
 
     def __del__(self):
         """Make sure the connection (pool) gets closed"""
-        try:
-            if isinstance(self._session, requests.Session):
-                self._session.close()
-        except AttributeError:
-            pass
+        if getattr(self, "_session", None) and isinstance(self._session, REQUESTS_SESSION):
+            self._session.close()
 
     def _build_session(self):
         self._session = requests.Session()
@@ -407,6 +404,10 @@ class Spotify:
     ):
         """ Get Spotify catalog information about an artist's albums
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `artist_albums(..., include_groups='...')` instead.
+
             Parameters:
                 - artist_id - the artist ID, URI or URL
                 - include_groups - the types of items to return. One or more of 'album', 'single',
@@ -452,6 +453,9 @@ class Spotify:
             identified artist. Similarity is based on analysis of the
             Spotify community's listening history.
 
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - artist_id - the artist ID, URI or URL
         """
@@ -683,6 +687,10 @@ class Spotify:
     ):
         """ Get full details of the tracks of a playlist.
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_items(playlist_id, ..., additional_types=('track',))` instead.
+
             Parameters:
                 - playlist_id - the playlist ID, URI or URL
                 - fields - which fields to return
@@ -755,18 +763,22 @@ class Spotify:
         )
 
     def user_playlist(self, user, playlist_id=None, fields=None, market=None):
-        warnings.warn(
-            "You should use `playlist(playlist_id)` instead",
-            DeprecationWarning,
-        )
-
         """ Gets a single playlist of a user
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist(playlist_id)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
                 - fields - which fields to return
         """
+        warnings.warn(
+            "You should use `playlist(playlist_id)` instead",
+            DeprecationWarning,
+        )
+
         if playlist_id is None:
             return self._get(f"users/{user}/starred")
         return self.playlist(playlist_id, fields=fields, market=market)
@@ -780,13 +792,12 @@ class Spotify:
         offset=0,
         market=None,
     ):
-        warnings.warn(
-            "You should use `playlist_tracks(playlist_id)` instead",
-            DeprecationWarning,
-        )
-
         """ Get full details of the tracks of a playlist owned by a user.
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_tracks(playlist_id)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -795,6 +806,10 @@ class Spotify:
                 - offset - the index of the first track to return
                 - market - an ISO 3166-1 alpha-2 country code.
         """
+        warnings.warn(
+            "You should use `playlist_tracks(playlist_id)` instead",
+            DeprecationWarning,
+        )
         return self.playlist_tracks(
             playlist_id,
             limit=limit,
@@ -847,6 +862,10 @@ class Spotify:
 
             Changes a playlist's name and/or public/private state
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_change_details(playlist_id, ...)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -868,6 +887,10 @@ class Spotify:
 
             Unfollows (deletes) a playlist for a user
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `current_user_unfollow_playlist(playlist_id)` instead.
+
             Parameters:
                 - user - the id of the user
                 - name - the name of the playlist
@@ -885,6 +908,10 @@ class Spotify:
 
             Adds tracks to a playlist
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_add_items(playlist_id, tracks)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -906,6 +933,10 @@ class Spotify:
 
             Adds episodes to a playlist
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_add_items(playlist_id, episodes)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -925,6 +956,10 @@ class Spotify:
 
             Replace all tracks in a playlist for a user
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_replace_items(playlist_id, tracks)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -949,6 +984,10 @@ class Spotify:
 
             Reorder tracks in a playlist from a user
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_reorder_items(playlist_id, ...)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -974,6 +1013,10 @@ class Spotify:
 
             Removes all occurrences of the given tracks from the given playlist
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_remove_all_occurrences_of_items(playlist_id, tracks)` instead.
+
             Parameters:
                 - user - the id of the user
                 - playlist_id - the id of the playlist
@@ -994,7 +1037,10 @@ class Spotify:
     ):
         """ This function is no longer in use, please use the recommended function in the warning!
 
-            Removes all occurrences of the given tracks from the given playlist
+            Removes specific occurrences of the given tracks from the given playlist
+
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
 
             Parameters:
                 - user - the id of the user
@@ -1007,8 +1053,8 @@ class Spotify:
                 - snapshot_id - optional id of the playlist snapshot
         """
         warnings.warn(
-            "You should use `playlist_remove_specific_occurrences_of_items"
-            "(playlist_id, tracks)` instead",
+            "You're using `user_playlist_remove_specific_occurrences_of_tracks(...)`, "
+            "which is marked as deprecated by Spotify.",
             DeprecationWarning,
         )
         plid = self._get_id("playlist", playlist_id)
@@ -1032,6 +1078,10 @@ class Spotify:
 
             Add the current authenticated user as a follower of a playlist.
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `current_user_follow_playlist(playlist_id)` instead.
+
             Parameters:
                 - playlist_owner_id - the user id of the playlist owner
                 - playlist_id - the id of the playlist
@@ -1049,6 +1099,10 @@ class Spotify:
 
             Check to see if the given users are following the given playlist
 
+            .. deprecated::
+            This method is deprecated and may be removed in a future version. Use
+            `playlist_is_following(playlist_id, user_ids)` instead.
+
             Parameters:
                 - playlist_owner_id - the user id of the playlist owner
                 - playlist_id - the id of the playlist
@@ -1259,10 +1313,20 @@ class Spotify:
         """
         return self.me()
 
-    def current_user_playing_track(self):
+    def current_user_playing_track(self, market=None, additional_types=("track",)):
         """ Get information about the current users currently playing track.
+
+            Parameters:
+                - market - An ISO 3166-1 alpha-2 country code or the
+                           string from_token.
+                - additional_types - list of item types to return.
+                                     valid types are: track and episode
         """
-        return self._get("me/player/currently-playing")
+        return self._get(
+            "me/player/currently-playing",
+            market=market,
+            additional_types=",".join(additional_types)
+        )
 
     def current_user_saved_albums(self, limit=20, offset=0, market=None):
         """ Gets a list of the albums saved in the current authorized user's
@@ -1568,6 +1632,9 @@ class Spotify:
     ):
         """ Get a list of Spotify featured playlists
 
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - locale - The desired language, consisting of a lowercase ISO
                   639-1 alpha-2 language code and an uppercase ISO 3166-1 alpha-2
@@ -1664,6 +1731,9 @@ class Spotify:
     ):
         """ Get a list of playlists for a specific Spotify category
 
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - category_id - The Spotify category ID for the category.
 
@@ -1701,6 +1771,9 @@ class Spotify:
             (at least one of `seed_artists`, `seed_tracks` and `seed_genres`
             are needed)
 
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - seed_artists - a list of artist IDs, URIs or URLs
                 - seed_tracks - a list of track IDs, URIs or URLs
@@ -1761,17 +1834,24 @@ class Spotify:
         return self._get("recommendations", **params)
 
     def recommendation_genre_seeds(self):
+        """ Get a list of genres available for the recommendations function.
+
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+        """
         warnings.warn(
             "You're using `recommendation_genre_seeds(...)`, "
             "which is marked as deprecated by Spotify.",
             DeprecationWarning,
         )
-        """ Get a list of genres available for the recommendations function.
-        """
         return self._get("recommendations/available-genre-seeds")
 
     def audio_analysis(self, track_id):
         """ Get audio analysis for a track based upon its Spotify ID
+
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - track_id - a track URI, URL or ID
         """
@@ -1785,6 +1865,10 @@ class Spotify:
 
     def audio_features(self, tracks=[]):
         """ Get audio features for one or multiple tracks based upon their Spotify IDs
+
+            .. deprecated::
+            This endpoint has been removed by Spotify and is no longer available.
+
             Parameters:
                 - tracks - a list of track URIs, URLs or IDs, maximum: 100 ids
         """
