@@ -9,11 +9,12 @@ import os
 import warnings
 from types import TracebackType
 
+import requests
 import urllib3
 
 import spotipy
 
-LOGGER = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 CLIENT_CREDS_ENV_VARS = {
     "client_id": "SPOTIPY_CLIENT_ID",
@@ -22,6 +23,9 @@ CLIENT_CREDS_ENV_VARS = {
     "redirect_uri": "SPOTIPY_REDIRECT_URI",
 }
 
+# workaround for garbage collection
+REQUESTS_SESSION = requests.Session
+
 
 def prompt_for_user_token(
     username=None,
@@ -33,15 +37,11 @@ def prompt_for_user_token(
     oauth_manager=None,
     show_dialog=False
 ):
-    warnings.warn(
-        "'prompt_for_user_token' is deprecated."
-        "Use the following instead: "
-        "    auth_manager=SpotifyOAuth(scope=scope)"
-        "    spotipy.Spotify(auth_manager=auth_manager)",
-        DeprecationWarning
-    )
-    """Prompt the user to login if necessary and returns a user token
-       suitable for use with the spotipy.Spotify constructor.
+    """ Prompt the user to login if necessary and returns a user token
+        suitable for use with the spotipy.Spotify constructor.
+
+        .. deprecated::
+            This method is deprecated and may be removed in a future version.
 
         Parameters:
             - username - the Spotify username. (optional)
@@ -53,6 +53,14 @@ def prompt_for_user_token(
             - oauth_manager - OAuth manager object. (optional)
             - show_dialog - If True, a login prompt always shows or defaults to False. (optional)
     """
+    warnings.warn(
+        "'prompt_for_user_token' is deprecated."
+        "Use the following instead: "
+        "    auth_manager=SpotifyOAuth(scope=scope)"
+        "    spotipy.Spotify(auth_manager=auth_manager)",
+        DeprecationWarning
+    )
+
     if not oauth_manager:
         if not client_id:
             client_id = os.getenv("SPOTIPY_CLIENT_ID")
@@ -64,7 +72,7 @@ def prompt_for_user_token(
             redirect_uri = os.getenv("SPOTIPY_REDIRECT_URI")
 
         if not client_id:
-            LOGGER.warning(
+            logger.warning(
                 """
                 You need to set your Spotify API credentials.
                 You can do this by setting environment variables like so:
@@ -166,8 +174,9 @@ class Retry(urllib3.Retry):
         if response:
             retry_header = response.headers.get("Retry-After")
             if self.is_retry(method, response.status, bool(retry_header)):
-                logging.warning("Your application has reached a rate/request limit. "
-                                f"Retry will occur after: {retry_header}")
+                retry_header = retry_header or 0
+                logger.warning("Your application has reached a rate/request limit. "
+                               f"Retry will occur after: {retry_header} s")
         return super().increment(method,
                                  url,
                                  response=response,
